require 'active_support/core_ext/string/inflections'

module Confer

  class ConfiguratorError         < StandardError; end
  class ConfiguratorNotFoundError < ConfiguratorError; end

  #
  # Public: Encapsulates the actual interaction with the remote server.
  #
  class Configurator

    #
    # Public: Find and instantiate a Configurator with the given name.
    #
    # name    - A String containing the name of the configurator to get.
    # options - A Hash of options to pass to the configurator.
    #
    # Raises
    #
    # ConfiguratorNotFoundError - If a configurator with the specified name can
    #                             not be located.
    # ConfiguratorError         - If the configurator was found byt cannot be
    #                             instantiated for some reason.
    #
    # Returns a Configurator instance if found.
    #
    def self.get(name, options = {})
      require "confer/configurators/#{name}"
      Confer::Configurators.const_get(name.camelize).new(options)
    rescue LoadError => e
      raise ConfiguratorNotFoundError.new("Configurator #{name} not found")
    rescue StandardError => e
      raise ConfiguratorError.new(e)
    end

    #
    # Public: Instantiates a new Configurator.  By itself this method doesn't
    # really do very much for the base class, however for subclasses it will
    # attempt to apply the options passed to attributes if they exist.
    #
    # options - A Hash of configurator specific options.
    #
    def initialize(options = {})
      options.apply_to self
    end

    #
    # Public: Query and return the remote configuration.
    #
    def query(connection); end

    #
    # Public: Verify the remote configuration against the Configurator attributes.
    #
    def verify(connection); end

    #
    # Public: Modify the remote configuration to match the Configurator attributes.
    #
    def apply(connection); end
  end

end
