require 'net/ssh'

module Confer

  #
  # Public: Encapsulates a connection to a remote host.
  #
  class Connection

    #
    # Public: Creates a new instance and immediately connects to the remote
    # host, yielding the instance to the given block.
    #
    # opts  - The options to pass to the new Connection instance.
    # block - The block to receive the new instance.
    #
    def self.start(opts = {}, &block)
      Connection.new(opts).start(&block)
    end

    #
    # Public: A String containing the remote hostname.
    #
    attr_accessor :host

    #
    # Public: A String containing the remote username.
    #
    attr_accessor :user

    #
    # Public: A Hash containing the SSH configuration options.
    #
    attr_accessor :config

    #
    # Public: Initialies a Connection instance.
    #
    # host - The host to connect to.
    #
    def initialize(opts = {})
      @config ||= {}
      opts.apply_to self
    end

    #
    # Public: Opens a connection to the remote host and yields self to the
    # given block for further interaction.  The connection will be closed once
    # the block has executed.
    #
    # block - The block to yield to.
    #
    def start(&block)
      Net::SSH.start(self.host, self.user, self.config) do |ssh|
        @ssh = ssh
        yield(self)
      end
    end

    #
    # Public: Execute a command on the remote host.
    #
    # command - The command to execute.
    # opts    - A hash of options:
    #           :shell - True if the command should be run via the remote shell.
    #
    # Returns the raw response from the command.
    #
    def exec!(command, opts = {})
      @ssh.exec! command
    end

  end

end
