require 'yaml'

module Confer

  class InventoryError         < StandardError;  end
  class InventoryNotFoundError < InventoryError; end
  class InventorySyntaxError   < InventoryError; end

  #
  # Public: Encapsulates an inventory, which is simply a list of hosts and the
  # credentials and transport mechanisms required to interact with them.
  #
  class Inventory

    #
    # Public: Loads an inventory from a YAML file.
    #
    # path - A String containing the path to the YAML file to load.
    #
    # Returns an Inventory instance.
    #
    def self.from_file(path)
      self.from_hash YAML.load File.open(path, 'r').read
    rescue Errno::ENOENT => e
      raise InventoryNotFoundError.new(e)
    rescue Psych::SyntaxError => e
      raise InventorySyntaxError.new(e)
    end

    #
    # Public: Loads an inventory from a Hash of hosts.
    #
    # array - A Hash of hosts and their options, indexed by name.
    #
    # Returns an Inventory instance.
    #
    def self.from_hash(hash)
      Inventory.new Hash[hash.map { |k, v| [k, Host.new(k, v)] }]
    end

    #
    # Public: A Hash of the hosts and their options in this Inventory, indexed
    # by name.
    #
    attr_accessor :hosts

    #
    # Public: Creates a new Inventory instance.
    #
    # hosts - A Hash of hosts their options in the inventory, indexed by name.
    #
    def initialize(hosts = {})
      @hosts = hosts
    end

    #
    # Internal: Encapsulates a single host in an Inventory.
    #
    class Host

      #
      # Public: A String containing the name of the host.
      #
      attr_reader :name

      #
      # Public: A Hash containing the host options.
      #
      attr_reader :options

      #
      # Internal: Iniitalize a new Host with configuration.
      #
      # args - An array containing the name of the host and a Hash containing
      #        the host options.
      #
      def initialize(*args)
        @options = args.extract_options!
        @name    = args.first
      end

    end
  end

end
