require 'yaml'

module Confer

  class RecipeError         < StandardError; end
  class RecipeNotFoundError < RecipeError;   end
  class RecipeSyntaxError   < RecipeError;   end

  #
  # Public: Encapsulates a recipe.  Usually instantiated by calling a method
  # from a mixed in module such as `Confer::Recipe::Yaml`.
  #
  class Recipe

    #
    # Public: Loads a recipe from a YAML file.
    #
    # path - A String containing the path to the YAML file to load.
    #
    # Returns a Recipe instance.
    #
    def self.from_file(path)
      self.from_array YAML.load File.open(path, 'r').read
    rescue Errno::ENOENT => e
      raise RecipeNotFoundError.new(e)
    rescue Psych::SyntaxError => e
      raise RecipeSyntaxError.new(e)
    end

    #
    # Public: Loads a recipe from an array of steps.
    #
    # array - An Array containing a Hash of options indexed by name for each
    #         step in the recipe.
    #
    # Returns a Recipe instance.
    #
    def self.from_array(array)
      Recipe.new array.map { |e| Step.new(*e.first) }
    end

    #
    # Public: An Array of steps this recipe contains.
    #
    attr_reader :steps

    #
    # Public: Instantiate a new instance with the given steps.
    #
    # array - An Array of Hash objects defining the steps for this recipe.
    # opts  - A Hash of options.
    #
    def initialize(steps = [])
      @steps = steps
    end

    #
    # Internal: Encapsulates a single step in a recipe.
    #
    class Step

      #
      # Public: A String containing the name of the configurator.
      #
      attr_reader :configurator

      #
      # Public: A Hash containing the configurator options.
      #
      attr_reader :options

      #
      # Internal: Initialize a new Step with configuration.
      #
      # args - An array contaning the configurator name and a Hash containing
      #        the configurator options.
      #
      def initialize(*args)
        @options      = args.extract_options!
        @configurator = args.first
      end

    end

  end

end
